@@ -12,7 +12,6 @@
 #include "onyxd_int.h"
 #include "decodemv.h"
 #include "vpx_mem/vpx_mem.h"
-#include "vp8/common/recon.h"
 #include "vp8/common/findnearmv.h"
 
 #include <assert.h>
@@ -285,27 +284,37 @@ static void estimate_mb_mvs(const B_OVERLAP *block_overlaps,
                             int mb_to_top_edge,
                             int mb_to_bottom_edge)
 {
-    int i;
+    int row, col;
     int non_zero_count = 0;
     MV * const filtered_mv = &(mi->mbmi.mv.as_mv);
     union b_mode_info * const bmi = mi->bmi;
     filtered_mv->col = 0;
     filtered_mv->row = 0;
-    for (i = 0; i < 16; ++i)
+    mi->mbmi.need_to_clamp_mvs = 0;
+    for (row = 0; row < 4; ++row)
     {
-        /* Estimate vectors for all blocks which are overlapped by this type */
-        /* Interpolate/extrapolate the rest of the block's MVs */
-        estimate_mv(block_overlaps[i].overlaps, &(bmi[i]));
-        mi->mbmi.need_to_clamp_mvs = vp8_check_mv_bounds(&bmi[i].mv,
-                                                         mb_to_left_edge,
-                                                         mb_to_right_edge,
-                                                         mb_to_top_edge,
-                                                         mb_to_bottom_edge);
-        if (bmi[i].mv.as_int != 0)
+        int this_b_to_top_edge = mb_to_top_edge + ((row*4)<<3);
+        int this_b_to_bottom_edge = mb_to_bottom_edge - ((row*4)<<3);
+        for (col = 0; col < 4; ++col)
         {
-            ++non_zero_count;
-            filtered_mv->col += bmi[i].mv.as_mv.col;
-            filtered_mv->row += bmi[i].mv.as_mv.row;
+            int i = row * 4 + col;
+            int this_b_to_left_edge = mb_to_left_edge + ((col*4)<<3);
+            int this_b_to_right_edge = mb_to_right_edge - ((col*4)<<3);
+            /* Estimate vectors for all blocks which are overlapped by this */
+            /* type. Interpolate/extrapolate the rest of the block's MVs */
+            estimate_mv(block_overlaps[i].overlaps, &(bmi[i]));
+            mi->mbmi.need_to_clamp_mvs |= vp8_check_mv_bounds(
+                                                         &bmi[i].mv,
+                                                         this_b_to_left_edge,
+                                                         this_b_to_right_edge,
+                                                         this_b_to_top_edge,
+                                                         this_b_to_bottom_edge);
+            if (bmi[i].mv.as_int != 0)
+            {
+                ++non_zero_count;
+                filtered_mv->col += bmi[i].mv.as_mv.col;
+                filtered_mv->row += bmi[i].mv.as_mv.row;
+            }
         }
     }
     if (non_zero_count > 0)
@@ -384,11 +393,11 @@ static void estimate_missing_mvs(MB_OVERLAP *overlaps,
             mi->mbmi.partitioning = 3;
             mi->mbmi.segment_id = 0;
             estimate_mb_mvs(block_overlaps,
-                                mi,
-                                mb_to_left_edge,
-                                mb_to_right_edge,
-                                mb_to_top_edge,
-                                mb_to_bottom_edge);
+                            mi,
+                            mb_to_left_edge,
+                            mb_to_right_edge,
+                            mb_to_top_edge,
+                            mb_to_bottom_edge);
             ++mi;
         }
         mb_col = 0;
@@ -481,33 +490,6 @@ static void find_neighboring_blocks(MODE_INFO *mi,
     assert(i == 20);
 }
 
-/* Calculates which reference frame type is dominating among the neighbors */
-static MV_REFERENCE_FRAME dominant_ref_frame(EC_BLOCK *neighbors)
-{
-    /* Default to referring to "skip" */
-    MV_REFERENCE_FRAME dom_ref_frame = LAST_FRAME;
-    int max_ref_frame_cnt = 0;
-    int ref_frame_cnt[MAX_REF_FRAMES] = {0};
-    int i;
-    /* Count neighboring reference frames */
-    for (i = 0; i < NUM_NEIGHBORS; ++i)
-    {
-        if (neighbors[i].ref_frame < MAX_REF_FRAMES &&
-            neighbors[i].ref_frame != INTRA_FRAME)
-            ++ref_frame_cnt[neighbors[i].ref_frame];
-    }
-    /* Find maximum */
-    for (i = 0; i < MAX_REF_FRAMES; ++i)
-    {
-        if (ref_frame_cnt[i] > max_ref_frame_cnt)
-        {
-            dom_ref_frame = i;
-            max_ref_frame_cnt = ref_frame_cnt[i];
-        }
-    }
-    return dom_ref_frame;
-}
-
 /* Interpolates all motion vectors for a macroblock from the neighboring blocks'
  * motion vectors.
  */
@@ -527,14 +509,20 @@ static void interpolate_mvs(MACROBLOCKD *mb,
                                         {4,4}, {4,3}, {4,2}, {4,1}, {4,0},
                                         {4,-1}, {3,-1}, {2,-1}, {1,-1}, {0,-1}
                                       };
+    mi->mbmi.need_to_clamp_mvs = 0;
     for (row = 0; row < 4; ++row)
     {
+        int mb_to_top_edge = mb->mb_to_top_edge + ((row*4)<<3);
+        int mb_to_bottom_edge = mb->mb_to_bottom_edge - ((row*4)<<3);
         for (col = 0; col < 4; ++col)
         {
+            int mb_to_left_edge = mb->mb_to_left_edge + ((col*4)<<3);
+            int mb_to_right_edge = mb->mb_to_right_edge - ((col*4)<<3);
             int w_sum = 0;
             int mv_row_sum = 0;
             int mv_col_sum = 0;
             int_mv * const mv = &(mi->bmi[row*4 + col].mv);
+            mv->as_int = 0;
             for (i = 0; i < NUM_NEIGHBORS; ++i)
             {
                 /* Calculate the weighted sum of neighboring MVs referring
@@ -557,18 +545,12 @@ static void interpolate_mvs(MACROBLOCKD *mb,
                  */
                 mv->as_mv.row = mv_row_sum / w_sum;
                 mv->as_mv.col = mv_col_sum / w_sum;
-
-                mi->mbmi.need_to_clamp_mvs = vp8_check_mv_bounds(mv,
-                                                       mb->mb_to_left_edge,
-                                                       mb->mb_to_right_edge,
-                                                       mb->mb_to_top_edge,
-                                                       mb->mb_to_bottom_edge);
-            }
-            else
-            {
-                mv->as_int = 0;
-                mi->bmi[row*4 + col].as_mode = NEW4X4;
-                mi->mbmi.need_to_clamp_mvs = 0;
+                mi->mbmi.need_to_clamp_mvs |= vp8_check_mv_bounds(
+                                                            mv,
+                                                            mb_to_left_edge,
+                                                            mb_to_right_edge,
+                                                            mb_to_top_edge,
+                                                            mb_to_bottom_edge);
             }
         }
     }
@@ -581,7 +563,6 @@ void vp8_interpolate_motion(MACROBLOCKD *mb,
 {
     /* Find relevant neighboring blocks */
     EC_BLOCK neighbors[NUM_NEIGHBORS];
-    MV_REFERENCE_FRAME dom_ref_frame;
     int i;
     /* Initialize the array. MAX_REF_FRAMES is interpreted as "doesn't exist" */
     for (i = 0; i < NUM_NEIGHBORS; ++i)
@@ -594,13 +575,11 @@ void vp8_interpolate_motion(MACROBLOCKD *mb,
                                 mb_row, mb_col,
                                 mb_rows, mb_cols,
                                 mb->mode_info_stride);
-    /* Determine the dominant block type */
-    dom_ref_frame = dominant_ref_frame(neighbors);
-    /* Interpolate MVs for the missing blocks
-     * from the dominating MVs */
-    interpolate_mvs(mb, neighbors, dom_ref_frame);
+    /* Interpolate MVs for the missing blocks from the surrounding
+     * blocks which refer to the last frame. */
+    interpolate_mvs(mb, neighbors, LAST_FRAME);
 
-    mb->mode_info_context->mbmi.ref_frame = dom_ref_frame;
+    mb->mode_info_context->mbmi.ref_frame = LAST_FRAME;
     mb->mode_info_context->mbmi.mode = SPLITMV;
     mb->mode_info_context->mbmi.uv_mode = DC_PRED;
     mb->mode_info_context->mbmi.partitioning = 3;
@@ -611,9 +590,8 @@ void vp8_conceal_corrupt_mb(MACROBLOCKD *xd)
 {
     /* This macroblock has corrupt residual, use the motion compensated
        image (predictor) for concealment */
-    vp8_recon_copy16x16(xd->predictor, 16, xd->dst.y_buffer, xd->dst.y_stride);
-    vp8_recon_copy8x8(xd->predictor + 256, 8,
-                      xd->dst.u_buffer, xd->dst.uv_stride);
-    vp8_recon_copy8x8(xd->predictor + 320, 8,
-                      xd->dst.v_buffer, xd->dst.uv_stride);
+
+    /* The build predictor functions now output directly into the dst buffer,
+     * so the copies are no longer necessary */
+
 }
